﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace LiveNet.Contacts
{
    public class CUri : DictionaryBase
    {
        #region Private Data
        private bool m_changed = false;
        #endregion

        #region Constructors
        internal CUri()
        {
            BuildDictionary();
        }

        internal CUri(XmlNode xmlnode)
        {
            BuildDictionary();
            FromXml(xmlnode);
        }

        private void BuildDictionary()
        {
            this.Add("ID", "");
            this.Add("URIType", ""); 
            this.Add("Name", "");
            this.Add("Address", "");
        }
        #endregion

        #region Public Properties
        public int ID
        {
            get { return Convert.ToInt32(this["ID"]); }
            set { this["ID"] = value.ToString(); }
        }

        public URIType URIType
        {
            get { return (URIType)Enum.Parse(URIType.GetType(), this["URIType"]); }
            set { this["URIType"] = value.ToString(); }
        }

        public string Name
        {
            get { return this["Name"]; }
            set { this["Name"] = value.Substring(0, Math.Min(128, value.Length)); }
        }

        public string Address
        {
            get { return this["Address"]; }
            set { this["Address"] = value.Substring(0, Math.Min(2048, value.Length)); }
        }
       #endregion

        #region Public Methods
        public XmlDocument ToXml(OperationType op)
        {
            XmlDocument xmlDoc = new XmlDocument();
            string xml = "<URI>";
            foreach (DictionaryEntry de in this)
            {
                if ((op == OperationType.Update || op == OperationType.Create) && de.Key.ToString() == "ID") continue;
                xml = xml + "<" + de.Key + ">" + de.Value + "</" + de.Key + ">";
            }
            xml = xml + "</URI>";
            xmlDoc.LoadXml(xml);
            return xmlDoc;
        }
        #endregion

        #region Internal Methods
        public void FromXml(XmlNode xmlnode)
        {
            if (xmlnode == null) return;

            XmlNode node = xmlnode.SelectSingleNode("ID");
            if (node != null) this["ID"] = node.InnerText;
            node = xmlnode.SelectSingleNode("URIType");
            if (node != null) this["URIType"] = node.InnerText;
            node = xmlnode.SelectSingleNode("Name");
            if (node != null) this["Name"] = node.InnerText;
            node = xmlnode.SelectSingleNode("Address");
            if (node != null) this["Address"] = node.InnerText;
        }
        #endregion

        #region Dictionary Methods
        public String this[String key]
        {
            get { return ((String)Dictionary[key]); }
            set { Dictionary[key] = value; }
        }

        public ICollection Keys
        {
            get { return (Dictionary.Keys); }
        }

        public ICollection Values
        {
            get { return (Dictionary.Values); }
        }

        public void Add(String key, String value)
        {
            Dictionary.Add(key, value);
        }

        public bool Contains(String key)
        {
            return (Dictionary.Contains(key));
        }

        public void Remove(String key)
        {
            Dictionary.Remove(key);
        }

        protected override void OnInsert(Object key, Object value)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");

            if (value.GetType() != typeof(System.String))
                throw new ArgumentException("value must be of type String.", "value");
        }

        protected override void OnRemove(Object key, Object value)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");
        }

        protected override void OnSet(Object key, Object oldValue, Object newValue)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");

            if (newValue.GetType() != typeof(System.String))
                throw new ArgumentException("newValue must be of type String.", "newValue");
        }

        protected override void OnValidate(Object key, Object value)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");

            if (value.GetType() != typeof(System.String))
                throw new ArgumentException("value must be of type String.", "value");
        }
        #endregion
    }
}
