﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace LiveNet.Contacts
{
    public class Contact : DictionaryBase
    {
        #region Private variables
        private ProfileCollection m_profiles = new ProfileCollection();
        private EmailCollection m_emails = new EmailCollection();
        private PhoneCollection m_phones = new PhoneCollection();
        private LocationCollection m_locations = new LocationCollection();
        private URICollection m_uris = new URICollection();
        private bool m_changed = false;
        #endregion

        #region Constructors
        internal Contact()
        {
            BuildDictionary();
        }

        internal Contact(XmlNode xmlnode)
        {
            BuildDictionary();
            FromXml(xmlnode);
        }

        private void BuildDictionary()
        {
            this.Add("WindowsLiveID", "");
            this.Add("AutoUpdateEnabled", "");
            this.Add("AutoUpdateStatus", "");
            this.Add("PreferredEmail", "");
            this.Add("ID", Guid.Empty.ToString());
         }
        #endregion

        #region Public Properties
        #region Main Contact Info
        public Guid ID
        {
            get { return new Guid(this["ID"]); }
            set { this["ID"] = value.ToString(); }
        }

        public string WindowsLiveID
        {
            get { return this["WindowsLiveID"]; }
            set { this["WindowsLiveID"] = value; }
        }

        public string PreferredEmail
        {
            get { return this["PreferredEmail"]; }
            set { this["PreferredEmail"] = value; }
        }

        public bool AutoUpdateEnabled
        {
            get { return Convert.ToBoolean(this["AutoUpdateEnabled"]); }
            set { this["AutoUpdateEnabled"] = value.ToString(); }
        }

        public string AutoUpdateStatus
        {
            get { return this["AutoUpdateStatus"]; }
            set { this["AutoUpdateStatus"] = value; }
        }
        #endregion

        #region Profiles
        public ProfileCollection Profiles
        {
            get { return m_profiles; }
            set { m_profiles = value; }
        }
        #endregion

        #region Emails
        public EmailCollection Emails
        {
            get { return m_emails; }
            set { m_emails = value; }
        }
        #endregion

        #region Phones
        public PhoneCollection Phones
        {
            get { return m_phones; }
            set { m_phones = value; }
        }
        #endregion

        #region Locations
        public LocationCollection Locations
        {
            get { return m_locations; }
            set { m_locations = value; }
        }
        #endregion

        #region URIs
        public URICollection URIs
        {
            get { return m_uris; }
            set { m_uris = value; }
        }
        #endregion
        #endregion

        #region Public Methods
        public XmlDocument ToXml(OperationType op)
        {
            XmlDocument xmldoc = new XmlDocument();
            string xmlProfiles = Profiles.ToXml(op).InnerXml;

            string xmlContact = "";
            foreach (DictionaryEntry de in this)
            {
                if ((op == OperationType.Update || op == OperationType.Create) && (de.Key.ToString() == "ID" || de.Key.ToString() == "CID" || de.Key.ToString() == "AutoUpdateEnabled"
                    || de.Key.ToString() == "AutoUpdateStatus" || de.Key.ToString() == "PreferredEmail")) continue;
                xmlContact = xmlContact + "<" + de.Key + ">" + de.Value + "</" + de.Key + ">";
            }
            xmldoc.LoadXml(string.Format("<Contact>{0}{1}</Contact>", xmlContact, xmlProfiles));
            return xmldoc;
        }
        #endregion

        #region Internal Methods
        internal void FromXml(XmlNode xmlnode)
        {
            if (xmlnode == null) return;

            XmlNode node = xmlnode.SelectSingleNode("WindowsLiveID");
            if (node != null) this["WindowsLiveID"] = node.InnerText;
            node = xmlnode.SelectSingleNode("PreferredEmail");
            if (node != null) this["PreferredEmail"] = node.InnerText;
            node = xmlnode.SelectSingleNode("AutoUpdateEnabled");
            if (node != null) this["AutoUpdateEnabled"] = node.InnerText;
            node = xmlnode.SelectSingleNode("AutoUpdateStatus");
            if (node != null) this["AutoUpdateStatus"] = node.InnerText;
            node = xmlnode.SelectSingleNode("ID");
            if (node != null) this["ID"] = node.InnerText;
            Profiles.FromXml(xmlnode.SelectSingleNode("Profiles"));
            Emails.FromXml(xmlnode.SelectSingleNode("Emails"));
            Phones.FromXml(xmlnode.SelectSingleNode("Phones"));
            Locations.FromXml(xmlnode.SelectSingleNode("Locations"));
            URIs.FromXml(xmlnode.SelectSingleNode("URIs"));
        }
        #endregion

        #region Dictionary Methods
        public String this[String key]
        {
            get { return ((String)Dictionary[key]); }
            set { Dictionary[key] = value; }
        }

        public ICollection Keys
        {
            get { return (Dictionary.Keys); }
        }

        public ICollection Values
        {
            get { return (Dictionary.Values); }
        }

        public void Add(String key, String value)
        {
            Dictionary.Add(key, value);
        }

        public bool Contains(String key)
        {
            return (Dictionary.Contains(key));
        }

        public void Remove(String key)
        {
            Dictionary.Remove(key);
        }

        protected override void OnInsert(Object key, Object value)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");

            if (value.GetType() != typeof(System.String))
                throw new ArgumentException("value must be of type String.", "value");
        }

        protected override void OnRemove(Object key, Object value)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");
        }

        protected override void OnSet(Object key, Object oldValue, Object newValue)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");

            if (newValue.GetType() != typeof(System.String))
                throw new ArgumentException("newValue must be of type String.", "newValue");
        }

        protected override void OnValidate(Object key, Object value)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");

            if (value.GetType() != typeof(System.String))
                throw new ArgumentException("value must be of type String.", "value");
        }
        #endregion
    }
}
