﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace LiveNet.Contacts
{
    public class Tag : DictionaryBase
    {
        #region Private Data
        private ArrayList m_contactids = new ArrayList();
        private bool m_changed = false;
        #endregion

        #region Constructors
        internal Tag()
        {
            BuildDictionary();
        }

        internal Tag(XmlNode xmlnode)
        {
            BuildDictionary();
            FromXml(xmlnode);
        }

        private void BuildDictionary()
        {
            this.Add("ID", "");
            this.Add("Name", "");
        }
        #endregion

        #region Public Properties
        public Guid ID
        {
            get { return new Guid(this["ID"]); }
        }

        public string Name
        {
            get { return this["Name"]; }
            set { this["Name"] = value; Changed = true;  }
        }

        public ArrayList ContactIDs
        {
            get { return m_contactids; }
        }

        public bool Changed
        {
            get { return m_changed; }
            set { m_changed = value; }
        }
        #endregion

        #region ContactIDs
        public void LoadContactIDs(XmlNode xmlnode)
        {
            if (xmlnode == null) return;

            ContactIDs.Clear();
            XmlNodeList nodeContacts = xmlnode.SelectNodes("ContactID");
            foreach (XmlNode nodeContact in nodeContacts)
            {
                ContactIDs.Add(nodeContact.InnerText);
            }
        }
        #endregion

        #region Public Methods
        public XmlDocument ToXml(OperationType op)
        {
            XmlDocument xmlDoc = new XmlDocument();

            string xml = "<Tag>";
            foreach (DictionaryEntry de in this)
            {
                if ((op == OperationType.Update || op == OperationType.Create) && (de.Key.ToString() == "ID" || de.Key.ToString() == "Name")) continue;
                xml = xml + "<" + de.Key + ">" + de.Value + "</" + de.Key + ">";
            }
            xml = xml + "</Tag>";

            if (op != OperationType.Update)
            {
                xml = xml + "<ContactIDs>";
                foreach (string contactID in ContactIDs)
                {
                    xml = xml + "<ContactID>" + contactID + "</ContactID>";
                }
                xml = xml + "</ContactIDs>";
            }

            xmlDoc.LoadXml(xml);
            return xmlDoc;
        }
        #endregion

        #region Internal Methods
        public void FromXml(XmlNode xmlnode)
        {
            if (xmlnode == null) return;

            XmlNode node = xmlnode.SelectSingleNode("ID");
            if (node != null) this["ID"] = node.InnerText;
            node = xmlnode.SelectSingleNode("Name");
            if (node != null) this["Name"] = node.InnerText;
            node = xmlnode.SelectSingleNode("ContactIDs");
            LoadContactIDs(node);
        }
        #endregion

        #region Dictionary Methods
        public String this[String key]
        {
            get { return ((String)Dictionary[key]); }
            set { Dictionary[key] = value; }
        }

        public ICollection Keys
        {
            get { return (Dictionary.Keys); }
        }

        public ICollection Values
        {
            get { return (Dictionary.Values); }
        }

        public void Add(String key, String value)
        {
            Dictionary.Add(key, value);
            Changed = true;
        }

        public bool Contains(String key)
        {
            return (Dictionary.Contains(key));
        }

        public void Remove(String key)
        {
            Dictionary.Remove(key);
            Changed = true;
        }

        protected override void OnInsert(Object key, Object value)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");

            if (value.GetType() != typeof(System.String))
                throw new ArgumentException("value must be of type String.", "value");
        }

        protected override void OnRemove(Object key, Object value)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");
        }

        protected override void OnSet(Object key, Object oldValue, Object newValue)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");

            if (newValue.GetType() != typeof(System.String))
                throw new ArgumentException("newValue must be of type String.", "newValue");
        }

        protected override void OnValidate(Object key, Object value)
        {
            if (key.GetType() != typeof(System.String))
                throw new ArgumentException("key must be of type String.", "key");

            if (value.GetType() != typeof(System.String))
                throw new ArgumentException("value must be of type String.", "value");
        }
        #endregion
    }
}
