﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Collections;

namespace LiveNet.Contacts
{
    public class URICollection
    {
        #region Private Data
        private ArrayList uris = new ArrayList();
        #endregion

        #region Constructors
        internal URICollection()
        {
        }

        internal URICollection(XmlNode xmlnode)
        {
            FromXml(xmlnode);
        }
        #endregion

        #region Internal Methods
        internal void FromXml(XmlNode xmlnode)
        {
            if (xmlnode == null) return;

            XmlNodeList nodes = xmlnode.SelectNodes("Location");
            if (nodes != null)
            {
                foreach (XmlNode node in nodes)
                {
                    Add(new CUri(node));
                }
            }
        }

        internal void Add(CUri uri)
        {
            uris.Add(uri);
        }

        internal void Remove(CUri uri)
        {
            uris.Remove(uri);
        }

        internal void RemoveAll()
        {
            uris.Clear();
        }
        #endregion

        #region Public Properties
        public int Count
        {
            get { return uris.Count; }
        }

        public object[] ToArray
        {
            get { return uris.ToArray(); }
        }
        #endregion

        #region Public Methods
        public XmlDocument ToXml(OperationType op)
        {
            XmlDocument xmldoc = new XmlDocument();
            string xml = "<URIs>";
            foreach (CUri uri in uris)
            {
                xml = xml + uri.ToXml(op).InnerXml;
            }
            xml = xml + "</URIs>";
            xmldoc.LoadXml(xml);

            return xmldoc;
        }

        public IEnumerator GetEnumerator()
        {
            return uris.GetEnumerator();
        }

        public CUri Find(int uriID)
        {
            foreach (CUri uri in uris)
            {
                if (uri.ID == uriID) return uri;
            }
            return null;
        }

        public CUri New()
        {
            return new CUri();
        }
        #endregion
    }
}
